﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;

using Pixoneer.NXDL;       // 기존 함수 관련 기능
using Pixoneer.NXDL.NCC;   // 좌표 시스템 관련기능
using Pixoneer.NXDL.NIO;   // 파일 입출력 관련기능
using Pixoneer.NXDL.NRS;   // Remote Secsing 관련기능
using Pixoneer.NXDL.NGR;   // Graphic 관련 기능
using Pixoneer.NXDL.NXMap; // MapView 관련 기능

namespace XDL_MapView2
{
    /// <summary>
    /// MainWindow.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class MainWindow : Window
    {
        public XRasterIO m_RasterIO;            // 영상의 입출력을 담당할 객체 선언
        XRSLoadFile xrsFileInput1 = null;       // 영상 디스플레이용 파일 객체 생성
        XRSLoadFile xrsFileInput2 = null;       // 영상 처리용 파일 객체 생성

        double oldPixelSize = 0.0;      // 이전 화면의 픽셀 크기
        double oldWCX = 0.0;            // 이전 화면의 화면 센터 X값
        double oldWCY = 0.0;            // 이전 화면의 화면 센터 Y값

        public MainWindow()
        {
            InitializeComponent();

            String StrError;
            m_RasterIO = new XRasterIO();   // 객체 생성
            if (m_RasterIO.Initialize(out StrError) == false)   // 영상 입출력 객체 초기화
            {
                MessageBox.Show(StrError);
            }

            nxMapLayerComposites1.LayerVisible = true; // 영상레이어를 보이도록 속성을 변경
            nxMapLayerComposites2.LayerVisible = true; // 영상레이어를 보이도록 속성을 변경
        }
   
        private void openFileMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // 1. 파일 Open을 수행한다.
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "XDM file(*.xdm)|*.XDM||";
            openFileDialog.RestoreDirectory = true;

            Nullable<bool> result = openFileDialog.ShowDialog();
            if (result != true) return;

            string strFilePath = openFileDialog.FileName;

            // 2. FileDriverManger를 통해 XRSLoadFile을 생성한다.
            string strError;
            xrsFileInput1 = m_RasterIO.LoadFile(strFilePath, out strError, false, eIOCreateXLDMode.All_NoMsg);
            xrsFileInput2 = m_RasterIO.LoadFile(strFilePath, out strError, false, eIOCreateXLDMode.All_NoMsg);

            if (xrsFileInput1 == null || xrsFileInput2 == null)
            {
                MessageBox.Show("Cannot Open");
                return;
            }
         
            ShowImageOrigin();  // 실제 이미지를 왼쪽 화면에 도시하는 역할 

            // 새로운 파일을 열 때마다 오른쪽에 도시된 기존 이미지 파일을 삭제한다.
            // nxMapLayerComposites2개체에 존재하는 Composite를 관리하는 XDMCompManager를 꺼낸다. 
            XDMCompManager xdmCompManager = nxMapLayerComposites2.GetXDMCompManager();

            // 기존에 Composite가 존재한다면 모두 삭제한다.
            for (int i = 0; i < xdmCompManager.NumComp; i++)
            {
                XDMComposite comp = xdmCompManager.GetXDMCompositeAt(i);
                comp.Dispose();
            }
            xdmCompManager.RemoveXDMCompositeAll();
        }

        private void ShowImageOrigin()
        {
            // 기존에 생성되어 있는 Composite가 있다면 Lock을 걸어 도시 수행을 잠시 멈춘다. 
            nxMapLayerComposites1.Lock();

            // nxMapLayerComposites1개체에 존재하는 XDMCompManager를 꺼낸다. XDMCompManager가 실제로 Composite들의 관리를 수행한다. 
            XDMCompManager xdmCompManager = nxMapLayerComposites1.GetXDMCompManager();

            // 기존에 Composite가 존재한다면 모두 삭제한다.
            for (int i = 0; i < xdmCompManager.NumComp; i++)
            {
                XDMComposite comp = xdmCompManager.GetXDMCompositeAt(i);
                comp.Dispose();
            }
            xdmCompManager.RemoveXDMCompositeAll();

            // XDMComposite를 생성한다. .
            XDMComposite newComp = new XDMComposite();

            // 로딩된 파일로 부터 밴드의 갯수가 칼라로 그릴것인지 흑백으로 그릴것인지를 판단한다.
            // 만약 3개 미만인 경우는 Gray모드로 그리고, 3개 이상인 경우는 칼라로 그린다.
            int nNumBand = xrsFileInput1.NumBand;
            if (nNumBand < 3)   // 3개 미만의 밴드를 가지고 있는 경우
            {
                // Default로 0번째 밴드를 Gray모드로 그린다. 
                XDMBand band = xrsFileInput1.GetBandAt(0);

                // 밴드 이름을 그대로 Composite이름으로 설정
                newComp.Name = band.BandName;

                // Gray Mode로 그리는것을 설정
                newComp.Mode = eCompMode.Gray;

                // Composite의 0번에 Band설정, Gray모드에서는 0번 밴드만을 인식한다. 
                newComp.SetBand(ref band, 0);
                // 영상 Enhancement를 위해 Histogram의 범위 설정(정규분포 95% 영역)
                newComp.SetCutType(eCompCutType.Ct95, 0);
                // 영상 Enhancement를 위해 Histogram적용시 전체 영역에 대한 Histogram(Band)설정, Visible은 현재 영역
                newComp.SetStretchCoverage(eCompStretchCoverage.Band, 0);
                // 영상 Enhancement를 위해 Histogram의 Stretch형태 설정(Gaussian으로 설정)
                newComp.SetStretchType(eCompStretchType.Gaussian, 0);
            }
            else // 3개 이상의 밴드를 가지고 있는 경우 
            {
                XDMBand band1 = xrsFileInput1.GetBandAt(0);
                XDMBand band2 = xrsFileInput1.GetBandAt(1);
                XDMBand band3 = xrsFileInput1.GetBandAt(2);
                newComp.Name = xrsFileInput1.FileName; // 파일 이름을 Compoiste이름으로 설정

                newComp.Mode = eCompMode.RGB;  // RGB Mode로 그리는것을 설정
                newComp.SetBand(ref band1, 0); // Composite의 0번에 Band설정, Blue Channel설정 
                newComp.SetBand(ref band2, 1); // Composite의 1번에 Band설정, Green Channel설정
                newComp.SetBand(ref band3, 2); // Composite의 2번에 Band설정, Red Channel설정

                for (int i = 0; i < 3; i++)
                {
                    // 영상 Enhancement를 위해 Histogram의 범위 설정(정규분포 95% 영역)
                    newComp.SetCutType(eCompCutType.Ct95, i);
                    // 영상 Enhancement를 위해 Histogram적용시 전체 영역에 대한 Histogram(Band)설정, Visible은 현재 영역
                    newComp.SetStretchCoverage(eCompStretchCoverage.Band, i);
                    // 영상 Enhancement를 위해 Histogram의 Stretch형태 설정(Gaussian으로 설정)
                    newComp.SetStretchType(eCompStretchType.Gaussian, i);
                }
            }

            // 생성된 Composite를 XDMCompManger객체에 추가한다. 
            xdmCompManager.AddXDMComposite(ref newComp);

            // 전체 화면 보기를 설정한다.
            nxMapLayerComposites1.ZoomFit();

            // 화면 업데이트를 수행한다. 
            nxMapLayerComposites1.Invalidate();

            // 설정된 Lock를 해제한다. 
            nxMapLayerComposites1.UnLock();
        }

        private void spatialFilterToolsMenuItem_Click(object sender, RoutedEventArgs e)
        {
            if (xrsFileInput2 == null) return;

            XBandParamSpatialFilter param = new XBandParamSpatialFilter();
            param.SrcBand = xrsFileInput2.GetBandAt(0);                 // 영상처리를 위한 Source Band를 설정
            param.FilterMethod = eSpatialFilterMethod.Average;          // 필터 설정
            param.KernelSize = 3;                                       // Filter의 커널 사이즈 설정

            // XDMBandSpatialFiter 밴드를 생성하고 입력 변수를 설정
            XDMBandSpatialFilter bandSF = new XDMBandSpatialFilter();
            bandSF.SetInputParam(ref param);

            ShowProcessedImage(bandSF);
        }

        private void ShowProcessedImage(XDMBandSpatialFilter bandSF)
        {
            // 기존에 생성되어 있는 Composite가 있다면 Lock을 걸어 도시 수행을 잠시 멈춘다. 
            nxMapLayerComposites2.Lock();

            // nxMapLayerComposites2개체에 존재하는 XDMCompManager를 꺼낸다. XDMCompManager가 실제로 Composite들의 관리를 수행한다. 
            XDMCompManager xdmCompManager = nxMapLayerComposites2.GetXDMCompManager();

            // 기존에 Composite가 존재한다면 모두 삭제한다.
            for (int i = 0; i < xdmCompManager.NumComp; i++)
            {
                XDMComposite comp = xdmCompManager.GetXDMCompositeAt(i);
                comp.Dispose();
            }
            xdmCompManager.RemoveXDMCompositeAll();

            // XDMComposite를 생성한다. .
            XDMComposite newComp = new XDMComposite();

            // Spatial Filter 밴드를 Gray모드로 그린다. 
            XDMBand band = bandSF;

            // 밴드 이름을 그대로 Composite이름으로 설정
            newComp.Name = band.BandName;

            // Gray Mode로 그리는것을 설정
            newComp.Mode = eCompMode.Gray;

            // Composite의 0번에 Band설정, Gray모드에서는 0번 밴드만을 인식한다. 
            newComp.SetBand(ref band, 0);
            // 영상 Enhancement를 위해 Histogram의 범위 설정(정규분포 95% 영역)
            newComp.SetCutType(eCompCutType.Ct95, 0);
            // 영상 Enhancement를 위해 실시간 영상처리는 반드시 Visible로 설정
            newComp.SetStretchCoverage(eCompStretchCoverage.Visible, 0);
            // 영상 Enhancement를 위해 Histogram의 Stretch형태 설정(Gaussian으로 설정)
            newComp.SetStretchType(eCompStretchType.Gaussian, 0);

            // 생성된 Composite를 XDMCompManger객체에 추가한다. 
            xdmCompManager.AddXDMComposite(ref newComp);

            // 전체 화면 보기를 설정한다.
            nxMapLayerComposites2.ZoomFit();

            // 화면 업데이트를 수행한다. 
            nxMapLayerComposites2.Invalidate();

            // 설정된 Lock를 해제한다. 
            nxMapLayerComposites2.UnLock();
        }

        private bool nxMapLayerGPU1_OnOrthoRender(NXMapLayerGPU sender, NXMapViewDrawArgs DrawArgs)
        {
            // 마우스가 왼쪽 NXMapView컨트롤 위에 존재하는지 여부 확인
            if (!this.nxMapView1.IsUnderMouse())
                return default(bool);

            // Screen Bound를 얻는다. 
            double x1w, x2w, y1w, y2w;
            x1w = x2w = y1w = y2w = 0.0;
            nxMapView1.GetScreenBound(ref x1w, ref x2w, ref y1w, ref y2w);

            // 화면의 중심점을 계산한다.
            double wcx, wcy;
            wcx = (x1w + x2w) / 2.0;
            wcy = (y1w + y2w) / 2.0;

            // 현재 화면의 화소 공간 크기를 얻는다.
            double pixelSize = DrawArgs.PixelSize;

            // 이전 화소의 크기/센터와 현재 화면의 화소의 크기/센터값이 모두 동일한 경우 도시 성능을 위해 적용하지 않는다.
            if (oldPixelSize == DrawArgs.PixelSize &&
                oldWCX == wcx &&
                oldWCY == wcy) return true;

            // 왼쪽 NXMapView의 화면 도시 정보를 오른쪽 NXMapView컨트롤에 적용한다.
            nxMapView2.SetWorld(new XVertex2d(wcx, wcy), new XVertex2d(pixelSize, pixelSize), new XAngle(0));

            // 현재 픽셀 크기/화면 중심 좌표를 저장한다.
            oldPixelSize = DrawArgs.PixelSize;
            oldWCX = wcx;
            oldWCY = wcy;

            nxMapView2.RefreshScreen();

            return true;
        }
    }
}
